function [K,K3]=Dekodowanie(chromosom,Kschem,K3schem,RangeMac)
    for w=1:size(Kschem,'r') do
        for k=1:size(Kschem,'c') do
            if Kschem(w,k)==0 then
                K(w,k)=0;
            else
                Ngen=abs(Kschem(w,k)); 
                K(w,k)=sign(Kschem(w,k))*... 
                (RangeMac(Ngen,1)+(RangeMac(Ngen,2)-RangeMac(Ngen,1))*chromosom(Ngen));
            end
        end
    end

    for w=1:size(K3schem,'r') do
        for k=1:size(K3schem,'c') do
            if K3schem(w,k)==0 then
                K3(w,k)=0;
            else
                Ngen=abs(K3schem(w,k)); 
                K3(w,k)=sign(K3schem(w,k))*... 
                (RangeMac(Ngen,1)+(RangeMac(Ngen,2)-RangeMac(Ngen,1))*chromosom(Ngen));
            end
        end
    end
endfunction 

function F=FunCel(varargin)
    select argn(2)
    case 3 then 
        lambda=varargin(1); omega=varargin(2); wagi=varargin(3);
        omega=omega(1); lambda=lambda(1);
        K=zeros(2,2); K3=zeros(2,2);
    case 7 then 
        A=varargin(1); A3=varargin(2); C=varargin(3); 
        K=varargin(4); K3=varargin(5);
        omega=varargin(6); wagi=varargin(7);
    else
        error('Nieprawidłowa liczba argumentów wejściowych')
    end
    
    F=0;
    for i=1:size(omega,'*') do
        if argn(2)==7 then
            lambda=spec(A+omega(i)*A3+(K+omega(i)*K3)*C);
        end
        F1=0;
        for j=1:size(lambda,'*') do 
            if real(lambda(j))>=0 then
                F1=F1+wagi.f1.w;
            end
        end ; 
        F2=0; lzad=0;
        for n=1:size(wagi.f2.lzad,'*') do
            lzad=lzad+wagi.f2.lzad(n)*omega(i)^((n-1)*2);
        end
        for j=1:size(lambda,'*') do
            F2=F2+wagi.f2.w*abs(real(lambda(j)-lzad));
        end
        F3=0; lzad=0;
        for n=1:size(wagi.f3.lzad,'*') do
            lzad=lzad+wagi.f3.lzad(n)*omega(i)^((n-1)*2);
        end
        F3=F3+wagi.f3.w*abs(min(real(lambda))-lzad);
        F4=0;
        for j=1:size(lambda,'*') do
            F4=F4+wagi.f4.w*abs(imag(lambda(j)));
        end
        F5=wagi.f5.w*WskWzm(K+omega(i)*K3);
        F6=0;
        for j=1:size(lambda,'*') do 
            if real(lambda(j))>0 then        
                F6=F6+wagi.f6.w*real(lambda(j));
            end
        end
        F7=0; lzad=0;
        for n=1:size(wagi.f7.lzad,'*') do
            lzad=lzad+wagi.f7.lzad(n)*omega(i)^((n-1)*2);
        end
        for j=1:size(lambda,'*') do
            if real(lambda(j))>real(lzad) then
                F7=F7+wagi.f7.w*abs(real(lambda(j)-lzad));
            end
        end
        F8=0; lzad=0;
        for n=1:size(wagi.f8.lzad,'*') do
            lzad=lzad+wagi.f8.lzad(n)*omega(i)^((n-1)*2);
        end
        for j=1:size(lambda,'*') do
            if real(lambda(j))<real(lzad) then
                F8=F8+wagi.f8.w*abs(real(lambda(j)-lzad));
            end
        end
        F9=0; lzad=0;
        for n=1:size(wagi.f9.lzad,'*') do
            lzad=lzad+%i*wagi.f9.lzad(n)*omega(i)^((n-1)*2);
        end
        for j=1:size(lambda,'*') do
            if abs(imag(lambda(j)))>abs(imag(lzad)) then
                F9=F9+wagi.f9.w*abs(abs(imag(lambda(j)))-abs(imag(lzad)));
            end
        end
        F=F+F1+F2+F3+F4+F5+F6+F7+F8+F9;
    end
endfunction 

function NewGen=Reprodukcja(OldGen,sel)
    Nchr=size(OldGen.chromosom,'r'); 
    Ngen=size(OldGen.chromosom,'c'); 
    NewGen.chromosom(Nchr,Ngen)=0;
    NewGen.Fcel(Nchr)=0;
    if sel.typ=='rulet' then  
        SumFcel=sum(OldGen.Fcel(:)); 
        if SumFcel<>0 then
            for ii=1:Nchr do 
                Rulet(ii)=sum(OldGen.Fcel(1:ii))/SumFcel;
            end
        else
            Rulet(ii)=1; 
        end
        for ii=1:Nchr do 
            Chr1Rulet=rand(); Chr2Rulet=rand();                            
            Chr1num=1; while Chr1Rulet>Rulet(Chr1num) do Chr1num=Chr1num+1; end
            Chr2num=1; while Chr2Rulet>Rulet(Chr2num) do Chr2num=Chr2num+1; end
            Chr1(1:Ngen)=OldGen.chromosom(Chr1num,1:Ngen);
            Chr2(1:Ngen)=OldGen.chromosom(Chr2num,1:Ngen);
            if sel.cross=='av' then 
                Pcross=rand();
                NewGen.chromosom(ii,1:Ngen)=Chr1(1:Ngen)*Pcross+Chr2(1:Ngen)*(1-Pcross);
            elseif sel.cross=='1p' then 
                Pcross=round(rand()*Ngen);
                for jj=1:Ngen do
                    if jj>Pcross then
                        NewGen.chromosom(ii,jj)=Chr1(jj);
                    else
                        NewGen.chromosom(ii,jj)=Chr2(jj);
                    end
                end
            elseif sel.cross=='2p' then 
                Pcross1=round(rand()*Ngen);
                Pcross2=round(rand()*Ngen);
                for jj=1:Ngen do
                    if (jj>Pcross1)==(jj>Pcross2) then
                        NewGen.chromosom(ii,jj)=Chr1(jj);
                    else
                        NewGen.chromosom(ii,jj)=Chr2(jj);
                    end
                end
            else
                error('Unknown crossing-over method');
            end 
         end
    elseif sel.typ=='elit' then  
        [dummy,ind]=gsort(OldGen.Fcel,'g','i');                                       
        for ii=1:Nchr do
            NewGen.chromosom(ii,1:Ngen)=OldGen.chromosom(ind(ii),1:Ngen);
        end
        NchrElit=round(sel.proc/100*Nchr);
        if NchrElit<1 then NchrElit=1;
        elseif NchrElit>Nchr-1 then NchrElit=Nchr-1;
        end                                                                          
        for ii=NchrElit+1:Nchr do
            Chr1num=round(rand()*(NchrElit-1))+1; 
            Chr2num=round(rand()*(NchrElit-1))+1; 
            Chr1(1:Ngen)=NewGen.chromosom(Chr1num,1:Ngen);
            Chr2(1:Ngen)=NewGen.chromosom(Chr2num,1:Ngen);
            if sel.cross=='av' then 
                Pcross=rand();
                NewGen.chromosom(ii,1:Ngen)=Chr1(1:Ngen)*Pcross+Chr2(1:Ngen)*(1-Pcross);
            elseif sel.cross=='1p' then 
                Pcross=round(rand()*Ngen);
                for jj=1:Ngen do
                    if jj>Pcross then
                        NewGen.chromosom(ii,jj)=Chr1(jj);
                    else
                        NewGen.chromosom(ii,jj)=Chr2(jj);
                    end
                end
            elseif sel.cross=='2p' then 
                Pcross1=round(rand()*Ngen);
                Pcross2=round(rand()*Ngen);
                for jj=1:Ngen do
                    if (jj>Pcross1)==(jj>Pcross2) then
                        NewGen.chromosom(ii,jj)=Chr1(jj);
                    else
                        NewGen.chromosom(ii,jj)=Chr2(jj);
                    end
                end
            else
                error('Unknown crossing-over method');
            end 
        end
    elseif sel.typ=='turn' then 
        for ii=Nchr:-1:1 do
            T1(round(sel.proc/100*(Nchr-1))+1)=0; 
            T2=T1;                                
            nT1=1; 
            while nT1<=size(T1,'*') do
                T1(nT1)=round(rand()*(Nchr-1))+1;
                if nT1>1 then
                    if ~or(T1(1:nT1-1)==T1(nT1)) then
                        nT1=nT1+1;
                    end
                else
                    nT1=nT1+1;
                end
            end
            nT2=1;
            while nT2<=size(T2,'*') do
                T2(nT2)=round(rand()*(Nchr-1))+1;
                if nT2>1 then
                    if ~or(T2(1:nT2-1)==T2(nT2)) then
                        nT2=nT2+1;
                    end
                else
                    nT2=nT2+1;
                end
            end
            nQ1=find(OldGen.Fcel(T1)==min(OldGen.Fcel(T1)));
            nQ2=find(OldGen.Fcel(T2)==min(OldGen.Fcel(T2)));
            Chr1(1:Ngen)=OldGen.chromosom(T1(nQ1(1)),1:Ngen);
            Chr2(1:Ngen)=OldGen.chromosom(T2(nQ2(1)),1:Ngen);
            Pcross=rand();
            NewGen.chromosom(ii,1:Ngen)=Chr1(1:Ngen)*Pcross+Chr2(1:Ngen)*(1-Pcross);
        end
        
    else 
        error('Unknown selection method')
    end
endfunction 

function SetRandGen(varargin)
    rand('uniform'); 
    if argn(2)==0 then
        n=getdate("s"); 
        rand("seed",n);
    elseif argn(2)==1 then
        rand("seed",varargin(1)); 
    else
        error('Random generator: unknown option');        
    end
endfunction 

function NewGen=Mutacja(OldGen,pmut)
    Nchr=size(OldGen.chromosom,'r'); 
    Ngen=size(OldGen.chromosom,'c'); 
    NewGen.chromosom=OldGen.chromosom;
    NewGen.Fcel(Nchr)=0;
    for ii=1:Nchr do
        if pmut/100>rand() then
            NewGen.chromosom(ii,round(rand()*(Ngen-1))+1)=rand();  
        end
    end
    
endfunction 

function Fout=Odwzorowanie(Fin,sel);
    if sel.typ=='rulet' then
        Fout(1:size(Fin,'*'))=max(Fin)-Fin(1:size(Fin,'*'));
    elseif sel.typ=='elit' | sel.typ=='turn' then
        Fout=Fin;
    else
        error('Unknown selection method');
    end
endfunction 

function [K,K3,data]=Genet(A,A3,C,Kschem,K3schem,RangeMac,omega,wagi,sel,pmut,Nchr,Npok)
    wek1=[size(Kschem,'r'), size(K3schem,'r'), size(A,'r'), size(A3,'r'),...
         size(A,'c'), size(A3,'c'), size(C,'c')];
    wek2=[size(Kschem,'c'), size(K3schem,'c'), size(C,'r')];
    if max(wek1)<>min(wek1) | max(wek2)<>min(wek2) then
        error('Observer mtrices size mismatch')
    end
    wait=waitbar(0,'Random generation of an initial population');
    tic();
    Ngen=max([Kschem,K3schem]);
    Generation.chromosom=rand(Nchr,Ngen);
    Generation.Fcel(Nchr)=0;
    Generation.F(Nchr)=0;
    for jj=1:Nchr do
            [K,K3]=Dekodowanie(Generation.chromosom(jj,:),Kschem,K3schem,RangeMac); 
            Generation.F(jj)=FunCel(A,A3,C,K,K3,omega,wagi); 
        end
        Generation.Fcel=Odwzorowanie(Generation.F,sel);
    data.ParGenet.A=A;                  data.ParGenet.A3=A3;
    data.ParGenet.C=C;                  data.ParGenet.Kschem=Kschem;
    data.ParGenet.K3schem=K3schem;      data.ParGenet.RangeMac=RangeMac;
    data.ParGenet.omega=omega;          data.ParGenet.wagi=wagi;
    data.ParGenet.sel=sel;              data.ParGenet.pmut=pmut;
    data.ParGenet.Nchr=Nchr;            data.ParGenet.Npok=Npok;
    data.pokolenia(1).npok=0;
    data.pokolenia(1).Generation.Fcel=Generation.Fcel;
    data.pokolenia(1).Generation.F=Generation.F;
    data.pokolenia(1).Generation.chromosom=Generation.chromosom;
    for ii=1:Npok do
        waitbar(ii/Npok,'Processing generation no. '+string(ii)+' out of '+...
                string(Npok),wait)
        Generation=Reprodukcja(Generation,sel)
        Generation=Mutacja(Generation,pmut)
        for jj=1:Nchr do
            [K,K3]=Dekodowanie(Generation.chromosom(jj,:),Kschem,K3schem,RangeMac); 
            Generation.F(jj)=FunCel(A,A3,C,K,K3,omega,wagi); 
        end
        Generation.Fcel=Odwzorowanie(Generation.F,sel);
        data.pokolenia(ii+1).npok=ii;
        data.pokolenia(ii+1).Generation.Fcel=Generation.Fcel;
        data.pokolenia(ii+1).Generation.F=Generation.F;
        data.pokolenia(ii+1).Generation.chromosom=Generation.chromosom;
    end 
    NbestChr=find(Generation.F==min(Generation.F)); 
    NbestChr=NbestChr(1);
    [K,K3]=Dekodowanie(Generation.chromosom(NbestChr,:),Kschem,K3schem,RangeMac);
    data.ParGenet.TotalTime=toc(); 
    disp('Optimization completed. Total time '+string(data.ParGenet.TotalTime)+' s');
    close(wait)
endfunction 

function y=WskWzm(A)
    x=A.^2; y=0; W=size(A,'r');
    for w=1:W do
        y=y+sqrt(sum(x(w,:)))/W;
    end
endfunction

