function y=FalaNosnaPWM(t,fnos)
    //  sawtooth PWM carrier waveform
    T=1/fnos; // period
    tokr=modulo(t,T); 
    if tokr < (0.5*T) then 
        y=-1+4*fnos*tokr; 
    else
        y=3-4*fnos*tokr; 
    end
endfunction 

////////////////////////////////////////////////////////////////////////////////

function y=T(teta)
    // stator transformation matrix from phase coordinate system to Cartesian (ABC -> xy0)
    y=[ cos(teta),  cos(teta+4/3*%pi),  cos(teta+2/3*%pi);
       -sin(teta), -sin(teta+4/3*%pi), -sin(teta+2/3*%pi);
        sqrt(1/2),  sqrt(1/2),          sqrt(1/2)];
    y=sqrt(2/3)*y;
endfunction

////////////////////////////////////////////////////////////////////////////////

function dx=Dsilnik(x,u,mo,Rs,Rr,Ls,Lr,Lm,pb,J,todn)
    // right side of the differential equation of the motor (omega_x = 0)
    // x = [ysx ysy yrx yry omega]
    // u = [usx usy]
    // parameters Rs,Rr,Ls,Lr,Lm,J [p.u.]
    a=Lm/(Lm^2-Ls*Lr); b=Ls/(Lm^2-Ls*Lr); c=Lr/(Lm^2-Ls*Lr); 
    dx(1)=(c*Rs*x(1)-a*Rs*x(3)+u(1))/todn;
    dx(2)=(c*Rs*x(2)-a*Rs*x(4)+u(2))/todn;
    dx(3)=(b*Rr*x(3)-a*Rr*x(1)-x(4)*x(5))/todn;
    dx(4)=(b*Rr*x(4)-a*Rr*x(2)+x(3)*x(5))/todn;
    dx(5)=-pb/J*(pb*a*(x(3)*x(2)-x(4)*x(1))+mo)/todn;
endfunction

////////////////////////////////////////////////////////////////////////////////

function [y1,y2]=OutSilnik(x,Ls,Lr,Lm,pb)
    // output of the motor
    a=Lm/(Lm^2-Ls*Lr); c=Lr/(Lm^2-Ls*Lr);
    y1(1)=a*x(3)-c*x(1); // isx
    y1(2)=a*x(4)-c*x(2); // isy
    y1(3)=0;
    y2(1)=-a*pb*(x(3)*x(2)-x(4)*x(1)); // electromagnetic torque
endfunction

////////////////////////////////////////////////////////////////////////////////

function dxo=Dobserwator(xo,u,y,omega,Ao,A3o,Bo,Co1,K,K3,todn)
    // right side of the observer's differential equation
    dxo=((Ao+omega*A3o)*xo+Bo*u+(K+omega*K3)*(Co1*xo-y))/todn;
endfunction



